@@ -11,12 +11,8 @@ use crate::{
     error::Error,
 };
 use bytes::{BufMut, BytesMut};
-use futures::{
-    io::{AllowStdIo, AsyncRead, Cursor},
-    ready,
-    stream::Stream,
-    task::{Context, Poll},
-};
+use futures_core::{ready, Stream};
+use futures_util::io::{AllowStdIo, AsyncRead, Cursor};
 use http::{
     self,
     header::{self, HeaderName},
@@ -24,13 +20,13 @@ use http::{
 };
 use mime::{self, Mime};
 use std::{
-    borrow::BorrowMut,
     fmt::Display,
     fs::File,
     io::{self, Read},
     iter::Peekable,
     path::Path,
     pin::Pin,
+    task::{Context, Poll},
     vec::IntoIter,
 };
 
@@ -124,10 +120,8 @@ impl<'a> Stream for Body<'a> {
 
                     cx.waker().wake_by_ref();
 
-                    Poll::Pending
+                    Poll::Ready(Some(Ok(body.buf.split())))
                 } else {
-                    body.write_final_boundary();
-
                     // No current part, and no parts left means there is nothing
                     // left to write.
                     //
@@ -135,19 +129,37 @@ impl<'a> Stream for Body<'a> {
                 }
             }
             Some(ref mut read) => {
-                match ready!(Pin::new(read).poll_read(cx, body.buf.borrow_mut())) {
+                let uninit_chunk = body.buf.chunk_mut();
+                let uninit_chunk = std::ptr::slice_from_raw_parts_mut(
+                    uninit_chunk.as_mut_ptr(),
+                    uninit_chunk.len(),
+                );
+
+                // Safe because chunk_mut() returns an array of uninitialized bytes, we are
+                // transforming it into something that can be passed to poll_read.
+                let uninit_chunk = unsafe { uninit_chunk.as_mut().unwrap() };
+
+                match ready!(Pin::new(read).poll_read(cx, uninit_chunk)) {
                     // EOF: No data left to read. Get ready to move onto write the next part.
                     //
                     Ok(0) => {
                         body.current = None;
 
-                        cx.waker().wake_by_ref();
+                        body.write_final_boundary();
 
-                        Poll::Pending
+                        Poll::Ready(Some(Ok(body.buf.split())))
                     }
                     // Read some data.
                     //
-                    Ok(_bytes_read) => Poll::Ready(Some(Ok(body.buf.split()))),
+                    Ok(bytes_read) => {
+                        // Safe because poll_read returns the number of bytes that were read
+                        // into the buffer.
+                        unsafe {
+                            body.buf.advance_mut(bytes_read);
+                        }
+
+                        Poll::Ready(Some(Ok(body.buf.split())))
+                    }
                     // Error reading from underlying stream.
                     //
                     Err(e) => Poll::Ready(Some(Err(Error::ContentRead(e)))),
@@ -275,7 +287,7 @@ impl<'a> Form<'a> {
     ///
     /// ```
     /// use common_multipart_rfc7578::client::multipart;
-    /// use futures::io::Cursor;
+    /// use futures_util::io::Cursor;
     ///
     /// let bytes = Cursor::new("Hello World!");
     /// let mut form = multipart::Form::default();
@@ -418,7 +430,7 @@ impl<'a> Form<'a> {
     ///
     /// ```
     /// use common_multipart_rfc7578::client::multipart;
-    /// use futures::io::Cursor;
+    /// use futures_util::io::Cursor;
     ///
     /// let bytes = Cursor::new("Hello World!");
     /// let mut form = multipart::Form::default();
@@ -478,7 +490,7 @@ impl<'a> Form<'a> {
     ///
     /// ```
     /// use common_multipart_rfc7578::client::multipart;
-    /// use futures::io::Cursor;
+    /// use futures_util::io::Cursor;
     ///
     /// let bytes = Cursor::new("Hello World!");
     /// let mut form = multipart::Form::default();
@@ -674,7 +686,7 @@ mod tests {
     use super::{Body, Form};
     use crate::error::Error;
     use bytes::BytesMut;
-    use futures::TryStreamExt;
+    use futures_util::TryStreamExt;
     use std::{
         io::Cursor,
         path::{Path, PathBuf},
